#!/usr/bin/bash
set -e
OSM_OUT_DIR=counties
ZIPS_URL_BASE="http://www2.census.gov/geo/tiger/tiger2006se/"

LOCKFILE=""
UNZIP_DIR=""
function cleanup_err
{
	cleanup ERR
}
function cleanup_int
{
	cleanup INT
}
function cleanup
{
        #echo trap "$@"
	pushd $HOME
        (
         rm -f "$LOCKFILE" || true
	 rm "$UNZIP_DIR"/TGR* || true
	 rmdir "$UNZIP_DIR" || true
        ) 2> /dev/null
	popd
        true
}

function national_county_list
{
	LOCAL_COUNTY_LIST=~/.tiger.national.txt
	[ -e "$LOCAL_COUNTY_LIST" ] \
	|| wget -O "$LOCAL_COUNTY_LIST" \
		http://www.census.gov/geo/www/fips/fips65/data/national.txt
	if [ -e "$LOCAL_COUNTY_LIST" ]; then
		cat "$LOCAL_COUNTY_LIST";
	else
		echo could not download list of counties >&2
		exit 1;
	fi
}

function county_names_tiger_zips
{
	national_county_list \
	| grep -v ^State,FIPS \
	| awk -F, '{print $1 "/TGR" $2 $3 ".ZIP," $4 "," $5}'
}

county_names_tiger_zips \
| egrep "$1" \
| while read zipline; do
	trap cleanup_err ERR
	trap cleanup_int INT
	echo $zipline
	ZIP_PATH="${zipline/,*/}"
	COUNTY_NAME="$(echo ${zipline} | awk -F, '{print $2}')"
	COUNTY_TYPE="$(echo ${zipline} | awk -F, '{print $3}')"
	STATE="$(dirname $ZIP_PATH)"
	ZIP="$(basename $ZIP_PATH)"
	echo ct: "'$COUNTY_TYPE'"
	if [ "$COUNTY_TYPE" == "C7" ] && \
	   echo $zipline | egrep -q '(Baltimore|St. Louis|Bedford|Fairfax|Franklin|Richmond|Roanoke),C7$';
	then
		COUNTY_NAME="$COUNTY_NAME-City"
		echo CITY: "'$COUNTY_NAME'"
		exit;
	fi
	OSM_OUTFILE="$OSM_OUT_DIR/$STATE/$COUNTY_NAME.osm"
	if [ -e "$OSM_OUTFILE" ] || [ -e "$OSM_OUTFILE".gz ]; then
		echo $COUNTY_NAME, $STATE already processed
		continue
	else
		echo $COUNTY_NAME, $STATE needs to be processed
	fi
	mkdir -p "$OSM_OUT_DIR/$STATE"
	if ! lockfile -0 -r1 "$OSM_OUTFILE.lock";  then
		echo can not lock "$OSM_OUTFILE.lock"
		continue
	fi
	LOCKFILE="$(pwd)/$OSM_OUTFILE.lock"
	UNZIP_DIR=`pwd`/tmp$$
	mkdir -p $UNZIP_DIR
	pushd $UNZIP_DIR > /dev/null 2>&1
	echo downloading $COUNTY_NAME, $STATE...
	wget "$ZIPS_URL_BASE/$ZIP_PATH";
	echo done
	if ! [ -e "$ZIP" ]; then
		echo $ZIP does not exist
		popd > /dev/null
		cleanup 1
		continue
	fi
	echo `date`: $ZIP
	if ! unzip $ZIP > /dev/null 2>&1; then
		echo unzip failed
		popd > /dev/null
		cleanup 2
		continue
	fi
	popd > /dev/null
	DO_EXIT=false
	RUBY=ruby
	#RUBY=ruby-prof
	if ! nice -10 $RUBY tiger_to_osm.rb $UNZIP_DIR/; then
		DO_EXIT=true
	else
		XML=${ZIP/ZIP/osm.xml}
		mkdir -p "$(dirname "$OSM_OUTFILE")"
		mv $UNZIP_DIR/$XML "$OSM_OUTFILE"
		gzip --rsyncable "$OSM_OUTFILE"
	fi
	cleanup 3
	#$DO_EXIT && exit
done

# for i in */*.osm.xml.gz; do name="$(grep ${i/.osm.xml.gz/.ZIP} /home/dave/.fips-county-codes-zips | awk -F, '{print $2}')"; mv $i `dirname $i`/"$name".osm.gz ; done

